@@ -1,11 +1,13 @@
+from __future__ import absolute_import
 from ._version import version_info, __version__
 
-from . import style
-from .volume import *
+from . import styles
+from .widgets import *
 from .transferfunction import *
 from .examples import  *
 from . import datasets
 from . import embed
+from .pylab import *
 
 def _jupyter_nbextension_paths():
     return [{
