@@ -7,6 +7,11 @@ data_dir = os.path.expanduser("~/.ipyvolume/datasets")
 if not os.path.exists(data_dir):
 	os.makedirs(data_dir)
 
+try:
+    from urllib import urlretrieve # py2
+except ImportError:
+    from urllib.request import urlretrieve # py3
+
 
 osname = dict(darwin="osx", linux="linux", windows="windows")[platform.system().lower()]
 
@@ -35,8 +40,8 @@ class Dataset(object):
 				print("Download failed, exit code was: " +str(code) +" will try with curl")
 				code = os.system(self.download_command_curl())
 				if not os.path.exists(self.path):
-					print("Download failed again, exit code was: " +str(code) +" giving up")
-
+					print("Download failed again, exit code was: " +str(code) +" using urlretrieve")
+					self.download_urlretrieve()
 
 	def fetch(self):
 		self.download()
@@ -64,6 +69,8 @@ class Dataset(object):
 	def download_command_curl(self):
 		return "cd %s; curl -O -L %s" % (data_dir, self.url)
 
+	def download_urlretrieve(self):
+		urlretrieve(self.url, self.path)
 
 hdz2000    = Dataset("hdz2000")
 aquariusA2 = Dataset("aquarius-A2")
@@ -71,4 +78,4 @@ egpbosLCDM  = Dataset("egpbos-LCDM")
 zeldovich  = Dataset("zeldovich", density=False)
 animated_stream = Dataset("stream-animation", density=False, numpy=True)
 
-# low poly cat from: https://sketchfab.com/models/1e7143dfafd04ff4891efcb06949a0b4#
\ No newline at end of file
+# low poly cat from: https://sketchfab.com/models/1e7143dfafd04ff4891efcb06949a0b4#
